@@ -535,11 +535,7 @@ xfdesktop_settings_image_iconview_add(GtkTreeModel *model,
         name_utf8 = g_filename_to_utf8(name, name_length,
                                        NULL, NULL, NULL);
         if(name_utf8) {
-#if GLIB_CHECK_VERSION (2, 30, 0)
             size_string = g_format_size(file_size);
-#else
-            size_string = g_format_size_for_display(file_size);
-#endif
 
             /* Display the file name, file type, and file size in the tooltip. */
             name_markup = g_markup_printf_escaped(_("<b>%s</b>\nType: %s\nSize: %s"),
@@ -1037,8 +1033,9 @@ cb_xfdesktop_chk_cycle_backdrop_toggled(GtkCheckButton *button,
 }
 
 static gboolean
-xfdesktop_spin_icon_size_timer(GtkSpinButton *button)
+xfdesktop_spin_icon_size_timer(gpointer user_data)
 {
+    GtkSpinButton *button = user_data;
     XfconfChannel *channel = g_object_get_data(G_OBJECT(button), "xfconf-chanel");
 
     TRACE("entering");
@@ -1070,9 +1067,7 @@ cb_xfdesktop_spin_icon_size_changed(GtkSpinButton *button,
         timer_id = 0;
     }
 
-    timer_id = g_timeout_add(500,
-                             (GSourceFunc)xfdesktop_spin_icon_size_timer,
-                             button);
+    timer_id = g_timeout_add(500, xfdesktop_spin_icon_size_timer, button);
 
     g_object_set_data(G_OBJECT(button), "timer-id", GUINT_TO_POINTER(timer_id));
 }
@@ -1482,23 +1477,6 @@ suboptions_set_sensitive(GtkToggleButton *btn,
     gtk_widget_set_sensitive(box, gtk_toggle_button_get_active(btn));
 }
 
-static gint
-display_get_monitor_num(GdkDisplay *display, GdkMonitor *monitor)
-{
-    gint i;
-
-    g_return_val_if_fail(GDK_IS_DISPLAY(display), 0);
-    g_return_val_if_fail(GDK_IS_MONITOR(monitor), 0);
-
-    for(i=0; i<gdk_display_get_n_monitors(display); i++) {
-        if(monitor == gdk_display_get_monitor(display, i))
-            return i;
-    }
-
-    g_warning("unable to get the monitor number");
-    return 0;
-}
-
 static void
 cb_update_background_tab(WnckWindow *wnck_window,
                          gpointer user_data)
@@ -1525,7 +1503,7 @@ cb_update_background_tab(WnckWindow *wnck_window,
     window = gtk_widget_get_window(panel->image_iconview);
     display = gdk_window_get_display(window);
     monitor = gdk_display_get_monitor_at_window(display, window);
-    monitor_num = display_get_monitor_num(display, monitor);
+    monitor_num = xfdesktop_get_monitor_num(display, monitor);
     monitor_name = xfdesktop_get_monitor_name_from_gtk_widget(panel->image_iconview, monitor_num);
 
     /* Most of the time we won't change monitor, screen, or workspace so try
@@ -2138,7 +2116,7 @@ main(int argc, char **argv)
     GtkBuilder *gxml;
     gint screen;
     GError *error = NULL;
-    AppearancePanel *panel = g_new0(AppearancePanel, 1);
+    AppearancePanel *panel;
 
 #ifdef G_ENABLE_DEBUG
     /* do NOT remove this line. If something doesn't work,
@@ -2148,10 +2126,6 @@ main(int argc, char **argv)
 
     xfce_textdomain(GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
 
-#if !GLIB_CHECK_VERSION(2, 32, 0)
-    g_thread_init(NULL);
-#endif
-
     if(!gtk_init_with_args(&argc, &argv, "", option_entries, PACKAGE, &error)) {
         if(G_LIKELY(error)) {
             g_printerr("%s: %s.\n", G_LOG_DOMAIN, error->message);
@@ -2197,6 +2171,7 @@ main(int argc, char **argv)
     }
 
     channel = xfconf_channel_new(XFDESKTOP_CHANNEL);
+    panel = g_new0(AppearancePanel, 1);
 
     if(opt_enable_debug)
         xfdesktop_debug_set(TRUE);
