@@ -65,6 +65,8 @@
 #include <gtk/gtk.h>
 #include <gio/gio.h>
 
+#include <cairo-xlib.h>
+
 #ifdef ENABLE_DESKTOP_ICONS
 #include "xfdesktop-icon-view.h"
 #include "xfdesktop-window-icon-manager.h"
@@ -102,6 +104,7 @@ struct _XfceDesktopPrivate
     gint nworkspaces;
     XfceWorkspace **workspaces;
     gint current_workspace;
+    gboolean current_workspace_initialized;
 
     gboolean single_workspace_mode;
     gint single_workspace_num;
@@ -315,13 +318,13 @@ set_real_root_window_surface(GdkScreen *gscreen,
                              cairo_surface_t *surface)
 {
 #ifndef DISABLE_FOR_BUG7442
-    Window xid;
+    Pixmap pixmap_id;
     GdkDisplay *display;
     GdkWindow *groot;
     cairo_pattern_t *pattern;
 
     groot = gdk_screen_get_root_window(gscreen);
-    xid = GDK_WINDOW_XID(groot);
+    pixmap_id = cairo_xlib_surface_get_drawable (surface);
 
     display = gdk_screen_get_display(gscreen);
     gdk_x11_display_error_trap_push(display);
@@ -330,7 +333,7 @@ set_real_root_window_surface(GdkScreen *gscreen,
     gdk_property_change(groot,
             gdk_atom_intern("_XROOTPMAP_ID", FALSE),
             gdk_atom_intern("PIXMAP", FALSE), 32,
-            GDK_PROP_MODE_REPLACE, (guchar *)&xid, 1);
+            GDK_PROP_MODE_REPLACE, (guchar *)&pixmap_id, 1);
     /* and set the root window's BG surface, because aterm is somewhat lame. */
     pattern = cairo_pattern_create_for_surface(surface);
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
@@ -745,10 +748,13 @@ workspace_changed_cb(WnckScreen *wnck_screen,
     current_workspace = desktop->priv->current_workspace;
     new_workspace = xfce_desktop_get_current_workspace(desktop);
 
+    if(desktop->priv->current_workspace_initialized && new_workspace == current_workspace)
+        return;
     if(new_workspace < 0 || new_workspace >= desktop->priv->nworkspaces)
         return;
 
     desktop->priv->current_workspace = new_workspace;
+    desktop->priv->current_workspace_initialized = TRUE;
 
     XF_DEBUG("current_workspace %d, new_workspace %d",
              current_workspace, new_workspace);
@@ -1168,7 +1174,7 @@ xfce_desktop_realize(GtkWidget *widget)
     GTK_WIDGET_CLASS(xfce_desktop_parent_class)->realize(widget);
 
     gtk_window_set_title(GTK_WINDOW(desktop), _("Desktop"));
-
+    gtk_window_set_decorated(GTK_WINDOW(desktop), FALSE);
     gtk_widget_set_size_request(GTK_WIDGET(desktop), sw, sh);
     gtk_window_move(GTK_WINDOW(desktop), 0, 0);
 
@@ -1267,7 +1273,6 @@ xfce_desktop_unrealize(GtkWidget *widget)
 
 #ifndef DISABLE_FOR_BUG7442
     gdk_property_delete(groot, gdk_atom_intern("_XROOTPMAP_ID", FALSE));
-    gdk_property_delete(groot, gdk_atom_intern("ESETROOT_PMAP_ID", FALSE));
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
     gdk_window_set_background_pattern(groot, NULL);
 G_GNUC_END_IGNORE_DEPRECATIONS
@@ -1408,9 +1413,9 @@ xfce_desktop_delete_event(GtkWidget *w,
 
 #ifdef ENABLE_DESKTOP_ICONS
 static gboolean
-style_refresh_cb(gpointer *w)
+style_refresh_cb(gpointer user_data)
 {
-    XfceDesktop *desktop = XFCE_DESKTOP(w);
+    XfceDesktop *desktop = user_data;
     cairo_pattern_t *pattern;
     gdouble old_font_size;
 
@@ -1467,8 +1472,8 @@ xfce_desktop_style_updated(GtkWidget *w)
         g_source_remove(desktop->priv->style_refresh_timer);
 
     desktop->priv->style_refresh_timer = g_idle_add_full(G_PRIORITY_LOW,
-                                                         (GSourceFunc)style_refresh_cb,
-                                                         w,
+                                                         style_refresh_cb,
+                                                         desktop,
                                                          NULL);
 #endif
 }
@@ -1902,9 +1907,11 @@ xfce_desktop_popup_secondary_root_menu(XfceDesktop *desktop,
 }
 
 void
-xfce_desktop_refresh(XfceDesktop *desktop, gboolean advance_wallpaper)
+xfce_desktop_refresh(XfceDesktop *desktop,
+                     gboolean advance_wallpaper,
+                     gboolean all_monitors)
 {
-    gint i, current_workspace;
+    gint i, current_workspace, current_monitor_num = -1;
 
     TRACE("entering");
 
@@ -1919,10 +1926,19 @@ xfce_desktop_refresh(XfceDesktop *desktop, gboolean advance_wallpaper)
 
     current_workspace = xfce_desktop_get_current_workspace(desktop);
 
+    if(!all_monitors) {
+        GdkDisplay *display = gdk_screen_get_display(desktop->priv->gscreen);
+        current_monitor_num = xfdesktop_get_current_monitor_num(display);
+    }
+
     /* reload backgrounds */
     for(i = 0; i < xfce_desktop_get_n_monitors(desktop); i++) {
         XfceBackdrop *backdrop;
 
+        if(!all_monitors && current_monitor_num != i) {
+            continue;
+        }
+
         backdrop = xfce_workspace_get_backdrop(desktop->priv->workspaces[current_workspace], i);
 
         if(advance_wallpaper) {
@@ -1935,13 +1951,10 @@ xfce_desktop_refresh(XfceDesktop *desktop, gboolean advance_wallpaper)
             backdrop_changed_cb(backdrop, desktop);
         }
     }
-
-    /* If we're only advancing the wallpaper we can exit here */
-    if(advance_wallpaper)
-        return;
 }
 
-void xfce_desktop_arrange_icons(XfceDesktop *desktop)
+void
+xfce_desktop_arrange_icons(XfceDesktop *desktop)
 {
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
 
@@ -1951,3 +1964,22 @@ void xfce_desktop_arrange_icons(XfceDesktop *desktop)
     xfdesktop_icon_view_sort_icons(XFDESKTOP_ICON_VIEW(desktop->priv->icon_view));
 #endif
 }
+
+gboolean
+xfce_desktop_get_cycle_backdrop(XfceDesktop *desktop)
+{
+    gint           monitor_num;
+    GdkDisplay    *display;
+    XfceWorkspace *workspace;
+    XfceBackdrop  *backdrop;
+
+    g_return_val_if_fail(XFCE_IS_DESKTOP(desktop), FALSE);
+
+    display = gdk_screen_get_display(desktop->priv->gscreen);
+    monitor_num = xfdesktop_get_current_monitor_num(display);
+
+    workspace = desktop->priv->workspaces[desktop->priv->current_workspace];
+    backdrop = xfce_workspace_get_backdrop(workspace, monitor_num);
+
+    return xfce_backdrop_get_cycle_backdrop(backdrop);
+}
