@@ -137,7 +137,30 @@ struct _XfdesktopApplicationClass
     GApplicationClass parent;
 };
 
-
+const gchar *fallback_CSS =
+"XfdesktopIconView.view {"
+"	background: transparent;"
+"	color: @theme_selected_fg_color;"
+"	border-radius: 3px;"
+"}"
+"XfdesktopIconView.view:active {"
+"	background: alpha(@theme_selected_bg_color, 0.5);"
+"	text-shadow: 0 1px 1px black;"
+"}"
+"XfdesktopIconView.view .label {"
+"	text-shadow: 1px 1px 2px black;"
+"}"
+"XfdesktopIconView.view .label:active {"
+"	color: @theme_selected_fg_color;"
+"}"
+"XfdesktopIconView .rubberband {"
+"	background: alpha(@theme_selected_bg_color, 0.2);"
+"	border: 1px solid @theme_selected_bg_color;"
+"	border-radius: 0;"
+"}"
+"XfdesktopIconView:active {"
+"	color: @theme_selected_bg_color;"
+"}";
 
 G_DEFINE_TYPE(XfdesktopApplication, xfdesktop_application, G_TYPE_APPLICATION)
 
@@ -352,7 +375,7 @@ reload_idle_cb(gpointer data)
 
     /* reload all the desktop */
     if(app->desktop)
-        xfce_desktop_refresh(XFCE_DESKTOP(app->desktop), FALSE);
+        xfce_desktop_refresh(XFCE_DESKTOP(app->desktop), FALSE, TRUE);
 
     g_application_release(G_APPLICATION(app));
 
@@ -373,7 +396,7 @@ cb_xfdesktop_application_reload(GAction  *action,
 
     /* hold the app so it doesn't quit while a queue up a refresh */
     g_application_hold(g_application);
-    g_idle_add((GSourceFunc)reload_idle_cb, g_application);
+    g_idle_add(reload_idle_cb, g_application);
 }
 
 static void
@@ -395,7 +418,7 @@ cb_xfdesktop_application_next(GAction  *action,
 
     /* reload the desktop forcing the wallpaper to advance */
     if(app->desktop)
-        xfce_desktop_refresh(XFCE_DESKTOP(app->desktop), TRUE);
+        xfce_desktop_refresh(XFCE_DESKTOP(app->desktop), TRUE, TRUE);
 }
 
 static void
@@ -621,9 +644,50 @@ xfdesktop_application_startup(GApplication *g_application)
     G_APPLICATION_CLASS(xfdesktop_application_parent_class)->startup(g_application);
 }
 
+static void
+xfdesktop_application_theme_changed (GtkSettings *settings,
+                                     XfdesktopApplication *app)
+{
+    GtkCssProvider *provider = NULL;
+    static GtkCssProvider *custom_provider = NULL;
+    gchar *theme;
+    gchar *css;
+
+    g_object_get(settings, "gtk-theme-name", &theme, NULL);
+
+    provider = gtk_css_provider_get_named(theme, NULL);
+    css = gtk_css_provider_to_string (provider);
+
+    if (g_strrstr (css, "XfdesktopIconView") != NULL) {
+        DBG("XfdesktopIconView section found in theme %s", theme);
+        if (custom_provider != NULL) {
+            gtk_style_context_remove_provider_for_screen (gdk_screen_get_default (),
+                                                          GTK_STYLE_PROVIDER(custom_provider));
+            g_clear_object (&custom_provider);
+        }
+    } else {
+        DBG("XfdesktopIconView section not found in theme %s, setting our fallback", theme);
+        if (custom_provider != NULL) {
+            gtk_style_context_remove_provider_for_screen (gdk_screen_get_default (),
+                                                          GTK_STYLE_PROVIDER(custom_provider));
+            g_clear_object (&custom_provider);
+        }
+        custom_provider = gtk_css_provider_new();
+        gtk_css_provider_load_from_data(custom_provider,
+                                        fallback_CSS,
+                                        -1,
+                                        NULL);
+        gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
+                                                   GTK_STYLE_PROVIDER(custom_provider),
+                                                   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+    }
+    g_free(css);
+}
+
 static void
 xfdesktop_application_start(XfdesktopApplication *app)
 {
+    GtkSettings *settings;
     GdkDisplay *gdpy;
     GError *error = NULL;
     gchar buf[1024];
@@ -632,6 +696,10 @@ xfdesktop_application_start(XfdesktopApplication *app)
 
     g_return_if_fail(app != NULL);
 
+    settings = gtk_settings_get_default();
+    g_signal_connect (settings, "notify::gtk-theme-name", G_CALLBACK (xfdesktop_application_theme_changed), NULL);
+    xfdesktop_application_theme_changed (settings, app);
+
     /* stop autostart timeout */
     if(app->wait_for_wm_timeout_id != 0)
         g_source_remove(app->wait_for_wm_timeout_id);
